def crea_matrice_zero(n):
    Z = [None] * n
    for i in range(0, n):
        Z[i] = [0] * n
    return Z


def cammini_doppio_grafo(M):
    # definire C[k][i][j] come "esiste un percorso da i a j
    # passando al più per i nodi {0,...,k-1}"

    # Due possibilità:
    # Calcoliamo C[n][i][j] per il grafo 1
    # Calcoliamo C[n][i][j] per il grafo 2
    # combiniamo il risultato
    #
    # oppure:
    # calcoliamo contemporanemante C[n][i][j]
    # per entrambi i grafi tenendo l'informazione sui
    # percorsi di entrambi

    # valori nella matrice:
    # 0 = nessun percorso
    # 1 = percorso nel grafo 1
    # 2 = percorso nel grafo 2
    # 3 = percorso nel grafo 1 e 2
    n = len(M)
    # sequenza di n+1 matrici
    C = [None] * (n+1)
    C[0] = M
    # Caso base, k = 0, nessun nodo intermedio
    # C[0][i][i] = 3 perché esiste un percorso tra i a i
    # C[0][i][j] = M[i][j] per i != j
    for i in range(0, n):
        C[0][i][i] = 3
    for k in range(0, n):
        C[k+1] = crea_matrice_zero(n)
        for i in range(0, n):
            for j in range(0, n):
                # Calcolare C[k+1][i][j] usando:
                # - C[k][i][j]: se un percorso esiste, continua # ad esistere
                # - C[k][i][k] e C[k][k][j]
                connessi_ij = C[k][i][j]
                # se ci manca il percorso nel grafo 1
                if connessi_ij == 0 or connessi_ij == 2:
                    connessi_ik = C[k][i][k] == 1 or C[k][i][k] == 3
                    connessi_kj = C[k][k][j] == 1 or C[k][k][j] == 3
                    if connessi_ik and connessi_kj:
                        connessi_ij += 1
                if connessi_ij == 0 or connessi_ij == 1:
                    connessi_ik = C[k][i][k] == 2 or C[k][i][k] == 3
                    connessi_kj = C[k][k][j] == 2 or C[k][k][j] == 3
                    if connessi_ik and connessi_kj:
                        connessi_ij += 2
                C[k+1][i][j] = connessi_ij
    Conteggio = crea_matrice_zero(n)
    for i in range(0, n):
        for j in range(0, n):
            if C[n][i][j] == 0:
                Conteggio[i][j] = 0
            elif C[n][i][j] == 1 or C[n][i][j] == 2:
                Conteggio[i][j] = 1
            else:  # C[n][i][j] == 3
                Conteggio[i][j] = 2
    return Conteggio


M = [[0, 2, 3, 0], [0, 0, 0, 1], [1, 2, 0, 0], [3, 1, 0, 0]]
